package prefuse.controls;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import prefuse.Visualization;
import prefuse.data.tuple.BatchTupleSet;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import cytoPrefuse.CytoPrefuse;

public class PopupMenuControl extends ControlAdapter implements ActionListener {
	private static final String LABEL_VISIBLE = "labelVisible";
	private static final String ADD_CURRENT = "addCurrent";
	private static final String ADD_NEW = "addNew";
	private static final String REMOVE_CURRENT = "removeCurrent";

	private VisualItem activeItem;
	private BatchTupleSet activeSelection;
	private CytoPrefuse cytoPrefuse;
	private Visualization vis;
	public PopupMenuControl (CytoPrefuse cytoPrefuse, Visualization vis){
		this.cytoPrefuse = cytoPrefuse;
		this.vis = vis;
	}
	public void itemClicked (VisualItem item, MouseEvent e){
		if (SwingUtilities.isRightMouseButton(e)){
			JPopupMenu menu = new JPopupMenu();
		
			/*
			if (item instanceof NodeItem){
				activeItem = item;
				JMenuItem labelVisible;
				if (activeItem.getBoolean(CytoPrefuse.USER_INTEREST)) {
					labelVisible = new JMenuItem("Let label be hidden");					
				}else{
					labelVisible = new JMenuItem("Keep label visible");
				}
				labelVisible.setActionCommand(LABEL_VISIBLE);
				menu.add(labelVisible);
				labelVisible.addActionListener(this);
			}
			*/
			if (item instanceof NodeItem || item instanceof EdgeItem){
				 activeItem = item;
				 activeSelection = (BatchTupleSet) vis.getFocusGroup(CytoPrefuse.SELECTED);
				 String messagePrefix = "Add ";
				 if (activeSelection.getTupleCount()> 1){
					 messagePrefix += "selected items ";
				 }
				 messagePrefix += "to ";
				 if (cytoPrefuse.groupsManager.getNumGroups() > 0) {				 
					 JMenuItem addToGroup = new JMenuItem(messagePrefix + cytoPrefuse.groupsManager.getCurrentGroupName());
					 addToGroup.setActionCommand(ADD_CURRENT);
					 addToGroup.addActionListener(this);
					 menu.add(addToGroup);
					 
					 String removePrefix = "Remove ";					 
					 if (activeSelection.getTupleCount()> 1){
						 removePrefix += "selected items ";
					 }
					 removePrefix += "from ";
					 
					 JMenuItem removeFromGroup = new JMenuItem(removePrefix + cytoPrefuse.groupsManager.getCurrentGroupName());
					 removeFromGroup.setActionCommand(REMOVE_CURRENT);
					 removeFromGroup.addActionListener(this);
					 menu.add(removeFromGroup);
					 
					 menu.addSeparator();
						
				 }
				 
				 for (int i=0; i < cytoPrefuse.groupsManager.getNumGroups(); i++){
					 String groupName = cytoPrefuse.groupsManager.getGroupName(i);
					 if (!groupName.equals(cytoPrefuse.groupsManager.getCurrentGroupName())){
						 JMenuItem addOtherGroup = new JMenuItem (messagePrefix + groupName);
						 addOtherGroup.setActionCommand(groupName);
						 addOtherGroup.addActionListener(this);
						 menu.add(addOtherGroup);
					 }
				 }
				 if (cytoPrefuse.groupsManager.getNumGroups() > 1) {
					 menu.addSeparator();
				 }
						
				 
					 
				JMenuItem addToNewGroup = new JMenuItem (messagePrefix + "a new group");
	
				addToNewGroup.setActionCommand (ADD_NEW);
				addToNewGroup.addActionListener(this);
				menu.add(addToNewGroup);
				 
			}
			menu.show(e.getComponent(),e.getX(), e.getY());
			
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals(LABEL_VISIBLE)) {
			activeItem.setBoolean(CytoPrefuse.USER_INTEREST, !activeItem.getBoolean(CytoPrefuse.USER_INTEREST));
		}else if (e.getActionCommand().equals(ADD_CURRENT)){			
			 addItemsToCurrentGroup();
		}else if (e.getActionCommand().equals(ADD_NEW)) {
			String newName =cytoPrefuse.groupsManager.addGroup();
			addItemsToGroup(newName);
		}else if (e.getActionCommand().equals(REMOVE_CURRENT)) {
			removeItemsFromCurrentGroup();
		}else {
			addItemsToGroup (e.getActionCommand());
		}
	}
	private void addItemsToGroup(String group) {
		if (activeSelection.getTupleCount() > 0){
			 Set addSet = new HashSet();
			 Iterator it = activeSelection.tuples();
			 while (it.hasNext()){
				 addSet.add(it.next());
			 }
			 cytoPrefuse.groupsManager.addItemsToGroup(addSet, group);				 
		 }else{
			 cytoPrefuse.groupsManager.addItemToGroup(activeItem, group);
		 }
		
	}
	private void removeItemsFromCurrentGroup() {
		if (activeSelection.getTupleCount() > 0){
			 Set removeSet = new HashSet();
			 Iterator it = activeSelection.tuples();
			 while (it.hasNext()){
				 removeSet.add(it.next());
			 }
			 cytoPrefuse.groupsManager.removeItemsFromGroup(removeSet);				 
		 }else{
			 cytoPrefuse.groupsManager.removeItemFromGroup(activeItem);
		 }
	}
	private void addItemsToCurrentGroup() {
		if (activeSelection.getTupleCount() > 0){
			 Set addSet = new HashSet();
			 Iterator it = activeSelection.tuples();
			 while (it.hasNext()){
				 addSet.add(it.next());
			 }
			 cytoPrefuse.groupsManager.addItemsToGroup(addSet);				 
		 }else{
			 cytoPrefuse.groupsManager.addItemToGroup(activeItem);
		 }
	}



}
