package prefuse.controls;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Iterator;

import prefuse.Display;
import prefuse.Visualization;
import prefuse.data.Tuple;
import prefuse.data.event.TupleSetListener;
import prefuse.data.tuple.TupleSet;
import prefuse.util.GraphicsLib;
import prefuse.util.display.DisplayLib;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

public class ZoomViewNeighbours extends ControlAdapter implements TupleSetListener{
	 private boolean zoomedOut = false;
	 private Rectangle2D originalBounds = new Rectangle2D.Float();
	 private Rectangle2D neighbourBounds = new Rectangle2D.Float();
	 private double originalZoom;
	 Point2D point = new Point2D.Float();
	 private NodeItem zoomOutNode;
	 private NodeItem focusNode;
	 private Visualization vis;
	 public ZoomViewNeighbours( Visualization vis, TupleSet selectGroup){
		 selectGroup.addTupleSetListener(this);
		 this.vis = vis;
	 }
	 public void itemEntered(VisualItem item, MouseEvent e) {
		if (item instanceof NodeItem) {			
			zoomOutNode = (NodeItem) item;
			focusNode =(NodeItem)item;
		}
	 }

	 public void itemKeyPressed(VisualItem item, KeyEvent e) {
		  if ((e.getModifiers() & InputEvent.ALT_MASK) == InputEvent.ALT_MASK) {
			  Display d = (Display)e.getSource();
			  zoomOutNode = null;
			  originalBounds.setFrame(d.getDisplayX()/d.getScale(), d.getDisplayY()/d.getScale(), d.getWidth()/d.getScale(), d.getHeight()/d.getScale());
			  originalZoom = d.getScale();
			  if (item instanceof NodeItem) {
				  neighbourBounds.setFrame(focusNode.getBounds());	  
				  Iterator edgesIt = focusNode.edges();
				  while (edgesIt.hasNext() ){					  
					  EdgeItem edge = (EdgeItem) edgesIt.next();					  
					  NodeItem neighbour = edge.getAdjacentItem(focusNode);
					  Rectangle2D.union(neighbourBounds, neighbour.getBounds(), neighbourBounds);
				  }
				  
			  }else {
				  EdgeItem edge = (EdgeItem) item;
				  neighbourBounds = edge.getBounds();
			  }
			  zoomedOut = true; 
			  d.setHighQuality(false);		      
			  GraphicsLib.expand(neighbourBounds,100);			 
			  DisplayLib.fitViewToBounds(d, neighbourBounds, 500);
			  vis.runAfter("delayHighQuality", 500);
		  }
	  } 
	 
	 public void itemKeyReleased(VisualItem item,KeyEvent e) {
		 zoomOutHandler(e);
	 } 
	 public void keyReleased(KeyEvent e) {
		zoomOutHandler(e);
	
	 }
	 
	 private void zoomOutHandler(KeyEvent e){
		 if (zoomedOut) {
			 Display d = (Display)e.getSource();
		 	 /*
		 	  * yuck.  Nothing works.  Hopefully will be improved in the next release....
				if (d.isTranformInProgress()) {
					//Before calculating scale, make sure we cancel the current animation
					//so that we can get an accurate current scale reading.
					//Currently no cancel funtion, so just execute a tiny pan (which cancels the animation)
					System.out.println ("Hack");
					d.pan(0.0000001, 0.000000001);
				}
				*/
			 d.setHighQuality(false);	
			 if (zoomOutNode != null) {				 							
			 	double scale = originalZoom/d.getScale();
			 	point.setLocation(zoomOutNode.getX(), zoomOutNode.getY());
			 	d.animatePanAndZoomToAbs(point, scale, 1000);
			 }else {
				 DisplayLib.fitViewToBounds(d, originalBounds, 500);
			 }
			 vis.runAfter("delayHighQuality", 1000);
			 zoomedOut = false;
		} 
	 }

	 

	public void tupleSetChanged(TupleSet tset, Tuple[] added, Tuple[] removed) {
        for ( int i=0; i<added.length; ++i ) {
        	Object item = added[i];
        	if (item instanceof NodeItem) {
        		zoomOutNode = (NodeItem) item;
        		break;
        	}
        }     
	}
}